// ============================================================================
// Project:	A Tiny DBMS (Exercise for Course "DBMS-Implementation")
// Filename:	block.cpp
// Purpose:	Blocks - Units of e.g. 8K in data files.
// Last Change:	03.12.2011
// Language:	C++
// Author:	Stefan Brass
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2011 by Stefan Brass
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "check.h"
#include "btype.h"
#include "block.h"


//=============================================================================
// Constructor, Destructor:
//=============================================================================


//-----------------------------------------------------------------------------
// Constructor (constructs empty/untyped block, e.g. for buffer):
//-----------------------------------------------------------------------------

block_c::block_c()
{
	// In this special case, the magic number is not only used in
	// debug mode, but always.
	// This permits a first test whether the block really belongs
	// to a file created by this program.

	// Initialize attributes:
	Magic = BLOCK_MAGIC;
	BType = BTYPE_EMPTY;
	HeadPartialWriteInd = BLOCK_PW_IND_A;
	FootPartialWriteInd = BLOCK_PW_IND_A;
	BlockNo = 0;

	// Initialize Data to 0:
	int *p = Data.IntData;
	int i = BLOCK_DATASIZE / sizeof(int);
	while(i-- > 0)
		*p++ = 0;
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

block_c::~block_c()
{
	// Make this object unusable:
	Magic = 0;
}


//=============================================================================
// Exported (Object) Methods:
//=============================================================================

//-----------------------------------------------------------------------------
// invalid: Check the integrity of this object.
//-----------------------------------------------------------------------------

	// The validity/integrity check is needed also in non-debug mode.
	// Blocks read from the disk must be checked.
	// A null pointer is returned if everything is ok, else an error msg.

str_t	block_c::invalid() const
{
	// Check object pointer:
	if(this == 0)
		return "NIL block_c object";

	// Check magic number:
	if(Magic != BLOCK_MAGIC)
		return "invalid block_c object (bad magic number)";

	// Check block type:
	str_t btype_msg = btype_invalid(BType);
	if(btype_msg)
		return btype_msg;

	// Partial Write Indicators must agree:
	if(HeadPartialWriteInd == BLOCK_PW_IND_A) {
		if(FootPartialWriteInd != BLOCK_PW_IND_A)
			return "Block was partially written or destroyed";
	}
	else if(HeadPartialWriteInd == BLOCK_PW_IND_B) {
		if(FootPartialWriteInd != BLOCK_PW_IND_B)
			return "Block was partially written or destroyed";
	}
	else {
		return "Partial Write Indicator is invalid";
	}

	// The block number is unsigned, so it cannot be negative.

	// No error found:
	return 0;
}


//-----------------------------------------------------------------------------
// Prepare a block for writing. Change partial write indicators.
//-----------------------------------------------------------------------------

void block_c::prepare_write()
{
	// Invert value in partial write indicators:
	if(HeadPartialWriteInd == BLOCK_PW_IND_A) {
		HeadPartialWriteInd = BLOCK_PW_IND_B;
		FootPartialWriteInd = BLOCK_PW_IND_B;
	}
	else {
		HeadPartialWriteInd = BLOCK_PW_IND_A;
		FootPartialWriteInd = BLOCK_PW_IND_A;
	}
}

