// ============================================================================
// Project:	A Tiny DBMS (Exercise for Course "DBMS-Implementation")
// Filename:	btype.h
// Purpose:	Enumeration type for different types/kinds of blocks
// Last Change:	20.02.2012
// Language:	C++
// Author:	Stefan Brass
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2011-2012 by Stefan Brass
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BTYPE_INCLUDED
#define BTYPE_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif


//=============================================================================
// Exported Types:
//=============================================================================

//-----------------------------------------------------------------------------
// btype_t: Enumeration type for different types/kinds of blocks.
//-----------------------------------------------------------------------------

typedef enum {
	BTYPE_INVALID,	/* Block not yet initialized */
	BTYPE_EMPTY,	/* E.g. free buffer or not assigned to segment */
	BTYPE_FILE_HEADER, /* File Header Block */
	BTYPE_REL_DATA	/* Data Block for a Relation */
} btype_t;


//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// btype_invalid: Function to check whether btype value is legal, NULL means ok
//-----------------------------------------------------------------------------

inline str_t btype_invalid(btype_t btype)
{
	switch(btype) {
	case BTYPE_INVALID:
		return "Block type is BTYPE_INVALID";
	case BTYPE_EMPTY:
	case BTYPE_FILE_HEADER:
	case BTYPE_REL_DATA:
		return 0;
	default:
		return "Unknown block type";
	}
}

//-----------------------------------------------------------------------------
// btype_name: Function to return the name of a block type (fro printing).
//-----------------------------------------------------------------------------

inline str_t btype_name(btype_t btype)
{
	switch(btype) {
	case BTYPE_INVALID:
		return "INVALID";
	case BTYPE_EMPTY:
		return "EMPTY";
	case BTYPE_FILE_HEADER:
		return "FILE_HEADER";
	case BTYPE_REL_DATA:
		return "REL_DATA";
	default:
		return "Unknown block type";
	}
}

//=============================================================================
// End of Include File:
//=============================================================================

#endif

