// ============================================================================
// Project:	A Tiny DBMS (Exercise for Course "DBMS-Implementation")
// Filename:	fhblk.cpp
// Purpose:	File Header Block [First Version, will be extended later]
// Last Change:	17.02.2012
// Language:	C++
// Author:	Stefan Brass
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2011-2012 by Stefan Brass
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "check.h"
#include "bno.h"
#include "fsize.h"
#include "err.h"
#include "btype.h"
#include "block.h"
#include "fhblk.h"

// #include <iostream>

//=============================================================================
// Constructor, Destructor:
//=============================================================================


//-----------------------------------------------------------------------------
// Constructor (constructs empty/untyped block, e.g. for buffer):
//-----------------------------------------------------------------------------

fhblk_c::fhblk_c() : block_c()
{
	// I get here the following warning:
	// fhblk.cpp: In constructor ‘fhblk_c::fhblk_c()’:
	// fhblk.cpp:58: warning: will never be executed
	// Even when the constructor is empty!
	// So it is a warning about code added by the compiler.
	// It doesn't suffice to do something only within this class.
	// One must call a function the compiler doesn't know.

	// Do anything to avoid warning:
#ifdef VER_AVOID_STRANGE_WARNING
	if(err_c::ok()) {
		for(int i = 1; i < 2; i++)
			Magic = Magic * i;
	}
#endif
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

fhblk_c::~fhblk_c()
{
	// Make this object unusable:
	Magic = 0;

}


//=============================================================================
// Exported (Object) Methods:
//=============================================================================

//-----------------------------------------------------------------------------
// invalid: Check the integrity of this object.
//-----------------------------------------------------------------------------

	// The validity/integrity check is needed also in non-debug mode.
	// Blocks read from the disk must be checked.
	// A null pointer is returned if everything is ok, else an error msg.

str_t	fhblk_c::invalid() const
{
	// Check object pointer:
	if(this == 0)
		return "NIL fhblk_c object";

	// Check magic number:
	if(Magic != BLOCK_MAGIC)
		return "invalid fhblk_c object (bad magic number)";

	// Check block type:
	if(BType != BTYPE_FILE_HEADER)
		return "Block is not a file header block";

	// Partial Write Indicators must agree:
	if(HeadPartialWriteInd == BLOCK_PW_IND_A) {
		if(FootPartialWriteInd != BLOCK_PW_IND_A)
			return "Block was partially written or destroyed";
	}
	else if(HeadPartialWriteInd == BLOCK_PW_IND_B) {
		if(FootPartialWriteInd != BLOCK_PW_IND_B)
			return "Block was partially written or destroyed";
	}
	else {
		return "Partial Write Indicator is invalid";
	}

	// No error found:
	return 0;
}


//-----------------------------------------------------------------------------
// Initialize header block (store size, reset possibly used block):
//-----------------------------------------------------------------------------

void fhblk_c::init(fsize_t size)
{
	// The size must be at least two (one header, one data block):
	CHECK(size > 1, "fhblk_c::init: size must be at least 2");

	// Set the block type:
	BType = BTYPE_FILE_HEADER;

	// Reset data part to 0:
	int *p = Data.IntData;
	int i = BLOCK_DATASIZE / sizeof(int);
	while(i-- > 0)
		*p++ = 0;

	// Set file size field:
	FHBLK_NUM_BLOCKS = size;

	// Set the total number of unused blocks:
	FHBLK_NUM_FREE = size - 1;

	// Set the file version number:
	FHBLK_FVERSION = 1;

}


//-----------------------------------------------------------------------------
// dump: Write contents of file header block in a readable way to alert file.
//-----------------------------------------------------------------------------

void fhblk_c::dump(str_t headline)
{
	// Check parameter:
	CHECK(headline != 0, "fhblk_c::dump: headline is NULL");

	// Check consistency of object:
	CHECK_VALID("fhblk_c::dump");

	// Write headline:
	err_c::dump_begin(headline);

	// Write number of blocks in file:
	err_c::dump_str("NUM_BLOCKS:   ");
	err_c::dump_uint(FHBLK_NUM_BLOCKS);
	err_c::dump_nl();

	// Write number of free blocks in file:
	err_c::dump_str("NUM_FREE:     ");
	err_c::dump_uint(FHBLK_NUM_FREE);
	err_c::dump_nl();

	// Write version number of the file:
	err_c::dump_str("FVERSION:     ");
	err_c::dump_int(FHBLK_FVERSION);
	err_c::dump_nl();

	// Mark end of dump, flush output:
	err_c::dump_end();
}


