// ============================================================================
// Project:	A Tiny DBMS (Exercise for Course "DBMS-Implementation")
// Filename:	perf.h
// Purpose:	Performance measurement functions, especially timer.
// Last Change:	28.01.2012
// Language:	C++
// Author:	Stefan Brass
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2010-2012 by Stefan Brass
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	PERF_INCLUDED
#define PERF_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// PERF_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long PERF_MAGIC = 0x5045521AL;	// 'PER\n'


//=============================================================================
// Class for Time Measurements:
//=============================================================================

class perf_c {
	public:


//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	perf_c();

	// Destructor:
	~perf_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// start: Start time measurement.
	void start();

	// stop: Stop time measurement.
	void stop();

	// realtime_msec: Time between last start() and stop() in milliseconds.
	long int realtime_msec();

	// cputime_msec: CPU time between last start() and stop() in millisecs.
	long int cputime_msec();


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity check:
	public:
	str_t	invalid() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "PERF_MAGIC".
#endif

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	perf_c(const perf_c& perf);		// Not implemented
	perf_c& operator=(const perf_c& perf);	// Not implemented

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

	// ID: Index in OS-dependent table of timer data (-1 if none free).
	int ID;


//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef perf_c *perf_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

