// ============================================================================
// Project:	A Tiny DBMS (Exercise for Course "DBMS-Implementation")
// Filename:	sbno.h
// Purpose:	Type for segment-relative block numbers.
// Last Change:	20.02.2012
// Language:	C++
// Author:	Stefan Brass
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	SBNO_INCLUDED
#define SBNO_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

//=============================================================================
// Exported Types:
//=============================================================================

//-----------------------------------------------------------------------------
// sbno_t: Type for segment-relative block numbers.
//-----------------------------------------------------------------------------

typedef VER_UINT sbno_t;
	// The first block in each segment is block number 0.
	// By using segment-relative block numbers, extents of segments
	// can be moved. Especially, even if a segment grows,
	// we can try to keep it in one piece of consecutive blocks
	// on the disk (one extent).
	// The mapping of segment-relative block numbers to absolute
	// block numbers is done in seg_c.
	// If we really have only one extent, the overhead is small.

//=============================================================================
// Exported Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// SBNO_NIL: Invalid block number, to be used as end marker for freelist.
//-----------------------------------------------------------------------------

const sbno_t SBNO_NIL = 0xFFFFFFFF;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

