// ============================================================================
// Project:	A Tiny DBMS (Exercise for Course "DBMS-Implementation")
// Filename:	str.h
// Purpose:	The String Datatype
// Last Change:	24.10.2011
// Language:	C++
// Author:	Stefan Brass
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1998-2011 by Stefan Brass
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	STR_INCLUDED
#define STR_INCLUDED

#ifdef	STR_MOD
#define	STR_X
#else
#define	STR_X	extern
#endif


//=============================================================================
// Exported Types:
//=============================================================================

//-----------------------------------------------------------------------------
// str_t: Constant strings.
//-----------------------------------------------------------------------------

typedef	const char *str_t;

#define STR_NIL ((str_t) 0)

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// str_len: Determine string length.
//-----------------------------------------------------------------------------

STR_X	int	str_len(register str_t str);

//-----------------------------------------------------------------------------
// str_cpy: Secure string copy.
//-----------------------------------------------------------------------------

STR_X	void	str_cpy(register char* buf, int len, register str_t str);

//-----------------------------------------------------------------------------
// str_cat: Secure string concatenation.
//-----------------------------------------------------------------------------

STR_X	void	str_cat(register char* buf, int len, register str_t str);

//-----------------------------------------------------------------------------
// str_eq: Compare two strings for equality.
//-----------------------------------------------------------------------------

STR_X	bool	str_eq(register str_t str1, register str_t str2);

//-----------------------------------------------------------------------------
// str_contains: Check whether character appears in string.
//-----------------------------------------------------------------------------

STR_X	bool	str_contains(register char c, register str_t str);

//-----------------------------------------------------------------------------
// str_error: System Error Message for an Error Number.
//-----------------------------------------------------------------------------

STR_X	void	str_error(int err_no, char *buf, int buf_size);

//-----------------------------------------------------------------------------
// STR_ERRMSG_LEN: Suggested Length of Buffer for System Error Messages.
//-----------------------------------------------------------------------------

#define STR_ERRMSG_LEN	80

//-----------------------------------------------------------------------------
// str_uint: Convert unsigned int to text. Returns number of characters.
//-----------------------------------------------------------------------------

STR_X	int str_uint(char *buf, int buf_len, unsigned int n);

//-----------------------------------------------------------------------------
// str_int: Convert int to text. Returns number of characters.
//-----------------------------------------------------------------------------

STR_X	int str_int(char *buf, int buf_len, int n);

//-----------------------------------------------------------------------------
// STR_INT_LEN: Suggested Length of Buffer for int Values.
//-----------------------------------------------------------------------------

#define STR_INT_LEN	11

//=============================================================================
// End of File:
//=============================================================================

#endif

