// ============================================================================
// Project:	A Tiny DBMS (Exercise for Course "DBMS-Implementation")
// Filename:	ver.h
// Purpose:	Version Settings
// Last Change:	29.05.2012
// Language:	C++
// Author:	Stefan Brass
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2001-2012 by Stefan Brass
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	VER_INCLUDED
#define VER_INCLUDED

//=============================================================================
// Basic Parameters:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_PROG_NAME: What is the name of this program?
//-----------------------------------------------------------------------------

#define VER_PROG_NAME "sbdb"

//-----------------------------------------------------------------------------
// VER_BLOCKSIZE: Size of a database block in bytes.
//-----------------------------------------------------------------------------

#define VER_BLOCKSIZE 8192

//-----------------------------------------------------------------------------
// VER_ALERT_FILE: Name of the file for error messages and warnings:
//-----------------------------------------------------------------------------

#define VER_ALERT_FILE "alert"

//-----------------------------------------------------------------------------
// VER_MAX_TIMERS: Number of timer_c objects existing at the same time.
//-----------------------------------------------------------------------------

#define VER_MAX_TIMERS 5

//-----------------------------------------------------------------------------
// VER_MAX_FILES: Size of the file_c object table, ID must be less than this.
//-----------------------------------------------------------------------------

#define VER_MAX_FILES 5

//-----------------------------------------------------------------------------
// VER_MAX_BUFS: Maximal number of buffers for blocks from data files.
//-----------------------------------------------------------------------------

#define VER_MAX_BUFS 2000

//-----------------------------------------------------------------------------
// VER_BUF_HASHSIZE: Should be ~25% larger than VER_MAX_BUFS, prime number.
//-----------------------------------------------------------------------------

#define VER_BUF_HASHSIZE 2503

//-----------------------------------------------------------------------------
// VER_ID_MAXLEN: Maximal length of identifiers (e.g. relation names).
//-----------------------------------------------------------------------------

#define VER_ID_MAXLEN 18 

//-----------------------------------------------------------------------------
// VER_MAX_COLS: Maximal number of columns of one relation.
//-----------------------------------------------------------------------------

#define VER_MAX_COLS 128

//=============================================================================
// Select an Operating System:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_UNIX/VER_WINDOWS: Select operating system
//-----------------------------------------------------------------------------

// We allow that these constants are already defined on the command line:
#ifndef VER_UNIX
#ifndef VER_WINDOWS

// Select here which version you want if the makefile does not define it:
// #define VER_WINDOWS
// #define VER_UNIX
// If none defined, we choose automatically below.

#endif // VER_WINDOWS
#endif // VER_UNIX

//-----------------------------------------------------------------------------
// VER_UNIX/VER_WINDOWS: Automatic choice of operating system
//-----------------------------------------------------------------------------

// Still not defined? Check for typical Microsoft Symbols, else choose UNIX:
#ifndef VER_UNIX
#ifndef VER_WINDOWS

#if defined(_MSC_VER) || defined(_WINDOWS_) || defined(WIN32)
#define VER_WINDOWS
#else
#define VER_UNIX
#endif

#endif // VER_WINDOWS
#endif // VER_UNIX

//=============================================================================
// Settings for Debugging:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_DEBUG: Check runtime assertions etc. (Debug Version).
//-----------------------------------------------------------------------------

#define VER_DEBUG

// Automatic choice for Visual C++:
#ifdef _DEBUG
#define VER_DEBUG
#endif

//-----------------------------------------------------------------------------
// VER_SIGNAL: Try to catch signals
//-----------------------------------------------------------------------------

#ifdef VER_UNIX
#define VER_SIGNAL
#endif


//=============================================================================
// Portability Definitions:
//=============================================================================

//-----------------------------------------------------------------------------
// VER_CS_ISO: Allow national characters according to ISO Latin 8859/1 code.
//-----------------------------------------------------------------------------

#ifdef VER_UNIX
#define VER_CS_ISO
#endif

//-----------------------------------------------------------------------------
// VER_CS_MSDOS: Allow national characters according to some MSDOS code.
//-----------------------------------------------------------------------------

// Only one of VER_CS_ISO and VER_CS_MSDOS may be defined.
// If none is defined, only ASCII characters are accepted.

// #define VER_CS_MSDOS

//-----------------------------------------------------------------------------
// Switch off Warning C4514: unreferenced inline function has been removed
//-----------------------------------------------------------------------------

// Microsoft Visual C++ 6.0 generates this warning in the highest warning
// level 4 when an include file defines an inline function not used in this
// source file. However, it might be used in a different source file which
// includes the same include file.

// #ifdef VER_WINDOWS
// #pragma warning (disable: 4514)
// #endif

//-----------------------------------------------------------------------------
// Add some code in subclass constructor to shut off g++ warnung:
//----------------------------------------------------------------------------

// fhblk.cpp: In constructor ‘fhblk_c::fhblk_c()’:
// fhblk.cpp:58: warning: will never be executed
// I got this warning with g++ even though the subclass constructor was empty,
// it only called the superclass constructor.
// I had to put some code inside (or switch the unreachable code warning off).
// This code has a slight performance penalty (it must call another function),
// so VER_AVOID_STRANGE_WARNING should be undefined for benchmarks.

#define VER_AVOID_STRANGE_WARNING

//-----------------------------------------------------------------------------
// Mark unused arguments so that the warning is not printed:
//-----------------------------------------------------------------------------

#ifdef __GNUC__
#define VER_UNUSED(ARG)	ARG __attribute__((unused))
#else
#define VER_UNUSED(ARG) ARG
#endif

//-----------------------------------------------------------------------------
// VER_BYTE: Data type for bytes in data blocks (sizeof must be 1).
//-----------------------------------------------------------------------------

typedef char VER_BYTE;

//-----------------------------------------------------------------------------
// VER_USHORT: Data type for 16 Bit numbers (unsigned), sizeof must be 2.
//-----------------------------------------------------------------------------

typedef unsigned short VER_USHORT;

//-----------------------------------------------------------------------------
// VER_INT: Data type for 32 Bit numbers, sizeof must be 4.
//-----------------------------------------------------------------------------

typedef int VER_INT;

//-----------------------------------------------------------------------------
// VER_UINT: Data type for unsigned 32 Bit numbers, sizeof must be 4.
//-----------------------------------------------------------------------------

typedef unsigned int VER_UINT;

//-----------------------------------------------------------------------------
// VER_SHORTOFFSET: How many shorts are n ints? Answer must be 2*n.
//-----------------------------------------------------------------------------

#define VER_SHORTOFFSET(N)	((N)*2)
	// The Data part of a block is a union of VER_BYTE, VER_USHORT, VER_INT
	// arrays. Sometimes different types are used in a block, but one must
	// make sure that the same memory location is not used with different
	// types.

//=============================================================================
// End of Include File:
//=============================================================================

#endif

